function MRAIDController() {
	var thisInstance = this;
	
	this.eventListeners = new Array(); // ['event'] = Array(listener1, listener2, ...)
	this.eventListenersLength = 0;
	
	/**** MRAID methods ****/
	this.getState = function() {
		return inova_android_mraid.getState();
	};
	
	this.getVersion = function() {
		return inova_android_mraid.getVersion();
	};
	
	this.getPlacementType = function() {
		return inova_android_mraid.getPlacementType();
	};
	
	this.isViewable = function() {
		return inova_android_mraid.isViewable();
	};
	
	this.open = function(url) {
		inova_android_mraid.open(url);
	};
	
	this.close = function() {
		inova_android_mraid.close();
	};
	
	this.useCustomClose = function(useCustom) {
		inova_android_mraid.useCustomClose(useCustom);
	};
	
	this.expand = function() {
		var countArgs = arguments.length;
		if (countArgs == 0) {
			inova_android_mraid.expand();
		} else if (countArgs == 1) {
			if (arguments[0] == undefined || arguments[0].length == 0) {
				inova_android_mraid.expand();
			} else {
				inova_android_mraid.expand(arguments[0]);
			}
		} else {
			thisInstance.notifyErrorEventListeners("Invalid number of arguments", "expand");
		}
	};
	
	this.resize = function() {
		inova_android_mraid.resize();
	};
	
	this.getDefaultPosition = function() {
		return JSON.parse(inova_android_mraid.getDefaultPosition());
	};
	
	this.getCurrentPosition = function() {
		return JSON.parse(inova_android_mraid.getCurrentPosition());
	};
	
	this.getMaxSize = function() {
		return JSON.parse(inova_android_mraid.getMaxSize());
	};
	
	this.getScreenSize = function() {
		return JSON.parse(inova_android_mraid.getScreenSize());
	};
	
	this.getExpandProperties = function() {
		return JSON.parse(inova_android_mraid.getExpandProperties());
	};
	
	this.setExpandProperties = function(properties) {
		inova_android_mraid.setExpandProperties(JSON.stringify(properties));
	};
	
	this.getResizeProperties = function() {
		return JSON.parse(inova_android_mraid.getResizeProperties());
	};
	
	this.setResizeProperties = function(properties) {
		inova_android_mraid.setResizeProperties(JSON.stringify(properties));
	};
	
	this.getOrientationProperties = function() {
		return JSON.parse(inova_android_mraid.getOrientationProperties());
	};
	
	this.setOrientationProperties = function(properties) {
		inova_android_mraid.setOrientationProperties(JSON.stringify(properties));
	};
	
	this.supports = function(value) {
		return inova_android_mraid.supports(value);
	};
	
	this.storePicture = function(uri) {
		inova_android_mraid.storePicture(uri);
	};
	
	this.createCalendarEvent = function(parameters) {
		inova_android_mraid.createCalendarEvent(JSON.stringify(parameters));
	};
	
	this.playVideo = function(uri) {
		inova_android_mraid.playVideo(uri);
	};
	
	/**
	 *  @param1 event - string, name of event to listen for
	 *  @param2 listener - function to execute
	 */
	this.addEventListener = function(event, listener) {
		if (thisInstance.listenerExists(event, listener)) {
			thisInstance.notifyErrorEventListeners("Listener is already registered. Ignoring call.", "addEventListener");
			inova_android_mraid.onAddEventListener();
			return;
		}
		
		if (thisInstance.eventListeners[event] == undefined) {
			thisInstance.eventListeners[event] = new Array();
			thisInstance.eventListenersLength++;
		}
		
		thisInstance.eventListeners[event].push(listener);
		inova_android_mraid.onAddEventListener();
	};
	
	/**
	 * If no listener function is specified, then all functions listening to the event will be removed. 
	 */
	this.removeEventListener = function(event, listener) {
		if (listener == undefined) {
			thisInstance.removeAllEventListeners(event);
			inova_android_mraid.onRemoveEventListener();
			return;
		}
		
		var arr = thisInstance.eventListeners[event];
		if (arr == undefined) {
			inova_android_mraid.onRemoveEventListener();
			return;
		}
		
		var removed = false;
		for (var i = 0; i < arr.length; i++) {
			if (arr[i] === listener) {
				if (arr.length == 1) {
					thisInstance.removeAllEventListeners(event);
				} else {
					arr.splice(i, 1);
				}
				removed = true;
				break;
			}
		}
		
		inova_android_mraid.onRemoveEventListener();
	};
	
	/**** end of MRAID methods ****/
	
	this.removeAllEventListeners = function(event) {
		if (thisInstance.eventListeners.hasOwnProperty(event)) {
			delete this.eventListeners[event];
			thisInstance.eventListenersLength--;
		}
	};
	
	this.listenerExists = function(event, listener) {
		var arr = thisInstance.eventListeners[event];
		if (arr === undefined || arr.length == 0) {
			return false;
		}
		
		for (var i = 0; i < arr.length; i++) {
			if (arr[i] === listener) {
				return true;
			}
		}
	};
		
	this.notifyEventListeners = function(event, callObject) {
		var arr = thisInstance.eventListeners[event];
		if (arr === undefined) {
			return;
		}
		
		for (var i = 0; i < arr.length; i++) {
			callObject.call(arr[i]);
		}
	};
	
	this.notifyReadyEventListeners = function() {
		thisInstance.notifyEventListeners("ready", new function() {
			this.call = function(ref) {
				ref();
			};
		});
	};
	
	this.notifyErrorEventListeners = function(message, error) {
		thisInstance.notifyEventListeners("error", new function() {
			this.call = function(ref) {
				ref(message, error);
			};
		});
	};
	
	this.notifyStateChangeEventListeners = function(state) {
		thisInstance.notifyEventListeners("stateChange", new function() {
			this.call = function(ref) {
				ref(state);
			};
		});
	};
	
	this.notifyViewableChangeEventListeners = function(onScreen) {
		thisInstance.notifyEventListeners("viewableChange", new function() {
			this.call = function(ref) {
				ref(onScreen);
			};
		});
	};
	
	this.notifySizeChangeEventListeners = function(width, height) {
		thisInstance.notifyEventListeners("sizeChange", new function() {
			this.call = function(ref) {
				ref(width, height);
			};
		});
	};
}
var mraid = new MRAIDController();