function VRVSDKController() {

    //Lopper reference so we can stop it.
    this.lopper;
    //Visibility flag
    this.visibility;
    //Standard callbacks - set, triggered once, removed from que
    this.standardCallbacks = new Array();

	//Multiple listeners for single event. Triggered multiple times until un-subscribed.
    this.scrollEventListeners = new Array();
    this.deviceAttributesListeners = new Array();
    this.visibilityChangeListeners = new Array();

    // Execution method called from Android side to execute standard callbacks. These are status response callbacks (success, error, result).
    this.executeStandardCallback = function(sJson, callbackId) {
        if (this.callbackExists(callbackId, this.standardCallbacks)) {
            if (sJson === undefined) {
                this.standardCallbacks[callbackId](undefined);
            } else {
                var json = JSON.parse(sJson);
                this.standardCallbacks[callbackId](json);
            }
            //Executed the callback. Clear it from references.
            this.removeCallback(callbackId, this.standardCallbacks);
        }
    };

    // Execution method called from android side to execute all subscribed scroll events listeners.
	this.executeScrollEventListeners = function(sJson) {
	    var json = JSON.parse(sJson);
    	for(var i = 0; i < this.scrollEventListeners.length; i++) {
    		this.scrollEventListeners[i](json);
    	}
    };

    // Execution method called from android side to execute all subscribed device attribution listeners.
    this.executeDeviceAttributesListeners = function(sJson) {
        var json = JSON.parse(sJson);
       	for(var i = 0; i < this.deviceAttributesListeners.length; i++) {
            this.deviceAttributesListeners[i](json);
        }
    };

    this.executeVisibilityChangeListeners = function(sJson) {
        var newVisibility;
        var json = JSON.parse(sJson);
        if (json.state == "visible") {
            newVisibility = true;
        } else {
            newVisibility = false;
        }

        if (this.visibility === undefined) {
            this.visibility = inova_android_vrvsdk.getDefaultVisibility();
        }

        if (this.visibility != newVisibility) {
            this.visibility = newVisibility;
            for(var i = 0; i < this.visibilityChangeListeners.length; i++) {
                this.visibilityChangeListeners[i](json);
            }
        }
    };

    //Execution method called from android side to execute the new bounds callback
    this.executeRequestNewBoundsCallback = function(status, error, sJsonAvailableBounds, callbackId) {
        if(this.callbackExists(callbackId, this.standardCallbacks)) {
            if(sJsonAvailableBounds === undefined) {
                this.standardCallbacks[callbackId](status, error, undefined);
            } else {
                var bounds = JSON.parse(sJsonAvailableBounds);
                this.standardCallbacks[callbackId](status, error, bounds);
            }
        //Executed the callback. Clear it from references.
        this.removeCallback(callbackId, this.standardCallbacks);
        }
    };

	//Supported methods through VRVSDKBridge

	this.canOpen = function(url, callback) {
	    var callbackId = this.generateCallbackKey();
	    this.standardCallbacks[callbackId] = callback;
		inova_android_vrvsdk.canOpen(url, callbackId);
	};
	
	this.share = function(service, text, url, imageUrl, callback) {
	    var callbackId = this.generateCallbackKey();
        this.standardCallbacks[callbackId] = callback;
		inova_android_vrvsdk.share(service, text, url, imageUrl, callbackId);
	};
	
	this.scheduleLocalNotification = function(text, date, callback) {
		var callbackId = this.generateCallbackKey();
        this.standardCallbacks[callbackId] = callback;
        if (text !== undefined && text !== null && date !== undefined && date !== null) {
            if (Object.prototype.toString.call(date) === '[object Date]') {
		        inova_android_vrvsdk.scheduleLocalNotification(text, date.toUTCString(), callbackId);
		    } else {
		         this.sendError(callbackId, "Argument date should be [object Date].");
		    }
		} else {
		    this.sendError(callbackId, "Arguments undefined/null.");
		}
	};

	this.setWindowTransparent = function(callback) {
		var callbackId = this.generateCallbackKey();
        this.standardCallbacks[callbackId] = callback;
		inova_android_vrvsdk.setWindowTransparent(callbackId);
	};

	this.userLocation = function(callback) {
	    var callbackId = this.generateCallbackKey();
        this.standardCallbacks[callbackId] = callback;
	    inova_android_vrvsdk.userLocation(callbackId);
	};

    this.getMediaPlaybackRequiresUserAction = function(callback) {
        var callbackId = this.generateCallbackKey();
        this.standardCallbacks[callbackId] = callback;
        inova_android_vrvsdk.getMediaPlaybackRequiresUserAction(callbackId);
    };

    this.setTouchIgnoreZones = function(zones, callback) {
        var callbackId = this.generateCallbackKey();
        this.standardCallbacks[callbackId] = callback;
        inova_android_vrvsdk.setTouchIgnoreZones(JSON.stringify(zones), callbackId);
    };

    this.sendScrollDataEvent = function(callback) {
        var callbackId = this.generateCallbackKey();
        this.standardCallbacks[callbackId] = callback;
        inova_android_vrvsdk.sendScrollDataEvent(callbackId);
    };

    this.getDeviceAttributes = function(callback) {
        var callbackId = this.generateCallbackKey();
        this.standardCallbacks[callbackId] = callback;
        inova_android_vrvsdk.getDeviceAttributes(callbackId);
    };

    this.getStyling = function(callback) {
        var callbackId = this.generateCallbackKey();
        this.standardCallbacks[callbackId] = callback;
        inova_android_vrvsdk.getStyling(callbackId);
    };

    this.registerCreative = function(attributes, callback) {
        var callbackId = this.generateCallbackKey();
        this.standardCallbacks[callbackId] = callback;
        var stringAttributes = "";
        if (attributes !== undefined && attributes !== null) {
            stringAttributes = JSON.stringify(attributes);
        }
        inova_android_vrvsdk.registerCreative(stringAttributes, callbackId);
    }

    this.requestNewBounds = function(bounds, animation, callback) {
        var callbackId = this.generateCallbackKey();
        this.standardCallbacks[callbackId] = callback;
        var stringBounds = "";
        var stringAnimation = "";
        if (bounds !== undefined && bounds !== null) {
            stringBounds = JSON.stringify(bounds);
        }
        if (animation !== undefined && animation !== null) {
            stringAnimation = JSON.stringify(animation);
        }
        inova_android_vrvsdk.requestNewBounds(stringBounds, stringAnimation, callbackId);
    }

    this.subscribeDeviceAttributesListener = function(listener, callback) {
        var callbackId = this.generateCallbackKey();
        this.standardCallbacks[callbackId] = callback;
        if (listener !== undefined && listener !== null) {
             this.deviceAttributesListeners.push(listener);
             inova_android_vrvsdk.subscribeDeviceAttributesListener(callbackId);
        } else {
            this.sendError(callbackId, "Cannot subscribe an undefined/null listener.");
        }
    };

    this.unsubscribeDeviceAttributesListener = function(listener, callback) {
        if (this.deviceAttributesListeners === undefined) {
            this.deviceAttributesListeners = new Array();
        }
        var array = this.deviceAttributesListeners;

        var callbackId = this.generateCallbackKey();
        this.standardCallbacks[callbackId] = callback;

        if (listener === undefined || listener === null) {
            this.sendError(callbackId, "Could not unsubscribe an undefined/null listener.");
        } else if (this.removeListenerIfExists(array, listener)) {
            this.deviceAttributesListeners = this.removeEmpty(array);
            inova_android_vrvsdk.unsubscribeDeviceAttributesListener(callbackId);
        } else {
            //Listener did not exist
            this.sendError(callbackId, "Could not unsubscribe. Listener did not exist!");
        }
    };

    this.subscribeScrollDataListener = function(listener, callback) {
        var callbackId = this.generateCallbackKey();
        this.standardCallbacks[callbackId] = callback;
        if (listener !== undefined && listener !== null) {
            this.scrollEventListeners.push(listener);
            inova_android_vrvsdk.subscribeScrollDataListener(callbackId);
        } else {
            this.sendError(callbackId, "Cannot set an undefined/null listener.");
        }
    };

    this.unsubscribeScrollDataListener = function(listener, callback) {
    	if (listener === undefined) {
    		return;
    	}

        if (this.scrollEventListeners === undefined) {
            this.scrollEventListeners = new Array();
        }
    	var array = this.scrollEventListeners;

    	var callbackId = this.generateCallbackKey();
        this.standardCallbacks[callbackId] = callback;

        if (listener === undefined || listener === null) {
            this.sendError(callbackId, "Could not unsubscribe an undefined/null listener.");
    	} else if (this.removeListenerIfExists(array, listener)) {
            this.scrollEventListeners = this.removeEmpty(array);
            inova_android_vrvsdk.unsubscribeScrollDataListener(callbackId);
    	} else {
    	    //Listener did not exist
    	    this.sendError(callbackId, "Could not unsubscribe. Listener did not exist!");
    	}
    };

     this.subscribeVisibilityChangeListener = function(listener, callback) {
         var callbackId = this.generateCallbackKey();
         this.standardCallbacks[callbackId] = callback;
         if (listener !== undefined && listener !== null) {

            //Start visibility lopper when first listener is subscribed
            if (this.visibilityChangeListeners.length == 0) {
                this.startTimer();
            }

            this.visibilityChangeListeners.push(listener);
            inova_android_vrvsdk.subscribeVisibilityChangeListener(callbackId);

            if (this.visibility === undefined) {
                this.visibility = inova_android_vrvsdk.getDefaultVisibility();
            }

            //Fire initial event with default visibility (forced event)
            var vis = this.visibility == true ? "visible" : "hidden";
            listener({state: vis});
         } else {
            this.sendError(callbackId, "Cannot set an undefined/null listener.");
         }
     };

     this.unsubscribeVisibilityChangeListener = function(listener, callback) {
        if (listener === undefined) {
            return;
        }

        if (this.visibilityChangeListeners === undefined) {
            this.visibilityChangeListeners = new Array();
        }
        var array = this.visibilityChangeListeners;

        var callbackId = this.generateCallbackKey();
        this.standardCallbacks[callbackId] = callback;
        if (listener === undefined || listener === null) {
            this.sendError(callbackId, "Could not unsubscribe an undefined/null listener.");
        } else if (this.removeListenerIfExists(array, listener)) {
            this.visibilityChangeListeners = this.removeEmpty(array);
            inova_android_vrvsdk.subscribeVisibilityChangeListener(callbackId);
        } else {
            //Listener did not exist
        	this.sendError(callbackId, "Could not unsubscribe. Listener did not exist!");
        }

        //Lopper stop if empty listener array
        if (this.visibilityChangeListeners.length == 0) {
            this.stopTimer();
        }
     };

     //VISIBILITY SUPPORT LOGIC

     /*
      * Events callback will change this. So the ad can check
      * if it has missed an event.
      */
     this.isVisible = function() {
        return this.inova_android_vrvsdk.isVisible();
     };

     /*
      * Starts our timer function.
      * Saves the looper instance so we can stop it
      * when its required.
      */
     this.startTimer = function() {
        this.looper = window.setInterval(this.timerFunction, 1000);
     };

     /*
      * Stops our timer function.
      * Stops it the using the saved instance of the looper that
      * was returned when starting it.
      */
     this.stopTimer = function() {
        window.clearInterval(this.looper);
     };

     /*
      * Timer function. Will tick at a set interval and
      * request the visibility from the AdSDK, executing
      * the visibility listeners.
      */
     this.timerFunction = function() {
         var vis = inova_android_vrvsdk.isVisible();
         var visibilityVal = (vis == true ? "visible" : "hidden");
         var result = JSON.stringify({ state: visibilityVal });
         vrvsdk.executeVisibilityChangeListeners(result);
     };

    //HELPER METHODS

    //Callback Helpers
    //Generates a callback key that is unique.
    this.generateCallbackKey = function() {
        return Date.now().toString();
    };

    // Removes the specified callback if it exists.
    this.removeCallback = function(callbackId, callbackArray) {
        if (this.callbackExists(callbackId, callbackArray)) {
            delete callbackArray[callbackId];
        }
    };

    //Checks if the callback instance exists.
    this.callbackExists = function(callbackId, callbackArray) {
        if (callbackArray !== undefined || callbackArray.length != 0) {
            if (callbackArray[callbackId] !== undefined) {
                return true;
            }
        }
    return false;
    };

    //Listener Helpers
    //Cycles listener array and removes it if it exists. Methods returns status.
    this.removeListenerIfExists = function(listenerArray, listener) {
        if (listenerArray !== undefined || listenerArray.length != 0) {
            for (var i = 0; i < listenerArray.length; i++) {
                if (listenerArray[i] === listener) {
                    delete listenerArray[i];
                    return true;
                }
            }
        }
        return false;
    };

    //No undefine values in array
     this.removeEmpty = function(array) {
         var newArray = new Array();
         for (var i = 0; i < array.length; i++) {
             if(array [i]  != undefined) {
                 newArray.push(array[i]);
             }
         }
         return newArray;
     };

    //Error in JS helper
    this.sendError = function(callbackId, reason) {
        var response = new Object;
        response.success = false;
        response.error = reason;
        this.executeStandardCallback(JSON.stringify(response), callbackId);
    };

}
var vrvsdk = new VRVSDKController();
